from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from wplay.utils import Logger
from wplay.utils.helpers import logs_path


#region LOGGER create
logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


async def blast(target):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        try:
            await target_search.search_and_select_target(page, target)
        except Exception as e:
            print(e)
            await page.reload()
            await target_search.search_and_select_target_without_new_chat_button(page,target)
    else:
        await target_select.manual_select_target(page)
    message : list[str] = io.ask_user_for_message_breakline_mode()
    number_of_messages : int = int(input("Enter the number of messages to blast: "))
    for _ in range(number_of_messages):
        logger.info("Blasting messages")
        await io.send_message(page, message)




# region IMPORTS
from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils import Logger
from wplay.utils.helpers import logs_path
# endregion


#region LOGGER create
logger : Logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


# region FOR SCRIPTING
async def get_last_seen_from_focused_target(page):
    logger.info("Writing target's status information")
    # await page.waitForSelector(whatsapp_selectors_dict['status'], visible = True)
    try:
        status : str = await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["last_seen"]}").getAttribute("title")')
        return status
    except:
        return '#status not found'
# endregion



__author__ = 'Alexandre Calil Martins Fonseca, github: xandao6'

# region IMPORTS
from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils import Logger
from wplay.utils.helpers import logs_path
# endregion


#region LOGGER create
logger : Logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


# region FOR SCRIPTING
def ask_user_for_message():
    logger.info("Input message")
    return str(input("Write your message: "))


def ask_user_for_message_breakline_mode():
    message = []  # type : list[str]
    i = 0
    print("Write your message ('Enter' to breakline)('.' alone to finish):")
    while True:
        message.append(str(input()))
        if message[i] == '.':
            message.pop(i)
            break
        elif message[i] == '...' or message[i] == '#_FILE':
            break
        i += 1
    return message


async def send_message(page, message):
    logger.info("Sending message")
    for i in range(len(message)):
        await page.type(
            whatsapp_selectors_dict['message_area'],
            message[i]
        )
        if isinstance(message, list):
            await page.keyboard.down('Shift')
            await page.keyboard.press('Enter')
            await page.keyboard.up('Shift')
    await page.keyboard.press('Enter')

#For Files

async def send_file(page):
    logger.info("Sending File")
    await page.click(whatsapp_selectors_dict['attach_file'])
    await page.click(whatsapp_selectors_dict['choose_file'])
    await page.waitForSelector(whatsapp_selectors_dict['send_file'],timeout=30000)
    await page.click(whatsapp_selectors_dict['send_file'])

# endregion



import argparse
import asyncio
import sys
import os

from pyfiglet import Figlet

from wplay.utils import kill_process
from wplay import onlinetracker
from wplay import messageblast
from wplay import messagetimer
from wplay import wchat
from wplay import savechat
from wplay import tgbot
from wplay import scheduleMessage
from wplay import changeAbout
from wplay import wnews
from wplay import get_media
from wplay.utils import Logger
from wplay.utils.helpers import logs_path


#region LOGGER create
logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion



def print_logo(text_logo):
    figlet = Figlet(font='slant')
    print(figlet.renderText(text_logo))


# parse positional and optional arguments
def get_arg_parser():
    parser = argparse.ArgumentParser(description = 'WhatsApp-play')
    parser.add_argument(
        "target",
        metavar="TARGET",
        type=str,
        default=None,
        nargs="?",
        help="contact or group name, optional , target can be selected manually except for saving chat")

    group = parser.add_mutually_exclusive_group(required = True)
    group.add_argument(
        "-wc",
        "--wchat",
        action = "store_true",
        help = "chatting from command line")

    group.add_argument(
        "-wb",
        "--wblast",
        action = "store_true",
        help = "message blast to a person")

    group.add_argument(
        "-wti",
        "--wtimer",
        action = "store_true",
        help = "send messages from time to time")

    group.add_argument(
        "-wt",
        "--wtrack",
        action = "store_true",
        help = "track online status of person")
    group.add_argument(
        "-wtb",
        "--wtgbot",
        action = "store_true",
        help = "sends tracking status to telegram bot")

    group.add_argument(
        "-pull",
        "--pull",
        action = "store_true",
        help = "save all chats, target is necessary")

    group.add_argument(
        "-sch",
        "--schedule",
        action = "store_true",
        help = "send the message at scheduled time")

    group.add_argument(
        "-wabt",
        "--wabout",
        action = "store_true",
        help = "Changes the about section"
    )

    group.add_argument(
        "-wnews",
        "--wnews",
        action = "store_true",
        help = "Get news in whatsapp group"
    )

    group.add_argument(
        "-wmedia",
        "--wgetmedia",
        action = "store_true",
        help = "Get profile photo of all your contacts"
    )

    # group.add_argument(
    #     "-wl",
    #     "--wlocation",
    #     action = "store_true",
    #     help = "finds the location of the person")

    return parser


# functions for different arguments
async def get_and_match_args(parser):
    args = parser.parse_args()
    if args.wtrack:
        await onlinetracker.tracker(args.target)

    elif args.wtgbot:
        tgbot.telegram_status(args.target)

    elif args.wchat:
        await wchat.chat(args.target)

    elif args.wblast:
        await messageblast.blast(args.target)

    elif args.wtimer:
        await messagetimer.msgTimer(args.target)

    elif args.pull:
        if args.target is None:
            parser.print_help()
            parser.exit()
        try:
            bID : int = int(sys.argv[3])
        except (IndexError, ValueError):
            bID : int = 0
        savechat.runMain('pull', str(args.target), bID)

    elif args.schedule:
        await scheduleMessage.schedule_message(args.target)

    elif args.wabout:
        await changeAbout.changeAbout()

    elif args.wnews:
        await wnews.get_news(args.target)

    elif args.wgetmedia:
        await get_media.get_all_media()

    # elif args.wlocation:
    #     loactionfinder.finder(args.target)


async def main():
    print_logo("wplay")
    parser = get_arg_parser()
    try:
        await get_and_match_args(parser)
        sys.exit(0)
    except KeyboardInterrupt:
        logger.error('User Pressed Ctrl+C')
        sys.exit(0)

try:
    asyncio.get_event_loop().run_until_complete(main())
except KeyboardInterrupt:
        logger.error('User Pressed Ctrl+C')
except AssertionError:
    try:
        for task in asyncio.all_tasks():
            task.cancel()
    except RuntimeError:
        exit()
    exit()
finally:
    kill_process.kill_child_processes(os.getpid())



import tkinter
from tkinter import filedialog
from telegram.ext import CommandHandler, Updater
from wplay.utils.helpers import data_folder_path

status_file_path = None


def start_tkinter():
    root_window = tkinter.Tk()
    root_window.withdraw()


def ask_where_are_the_status_file():
    print('Choose a status text file.')
    status_file_path = filedialog.askopenfile(
        initialdir = data_folder_path / 'tracking_data',
        title = 'Choose a status text file.',
        filetypes = (("text files", "*.txt"), ("all files", "*.*"))
    )
    if status_file_path == ():
        print("Error! Choose a status.")
        exit()
    return status_file_path


def startmessage(bot, update):
    chat_id : int = update.message.chat_id
    text : str = '''
        Hi, I am here to send all the messages you want to track online status in whatsapp :)
    '''
    bot.send_message(chat_id = chat_id, text = text)


def send_status(bot, update):
    # Display last updated online status message
    chat_id = update.message.chat_id
    try:
        f = open(status_file_path, 'r')
        file_data = f.readlines()
        text : Union[str , bytes] = file_data[len(file_data) - 1]
        bot.send_message(chat_id = chat_id, text = text)
    except:
        bot.send_message(chat_id = chat_id, text = 'oops! An error occurred')


def telegram_status(name):
    print(name)
    start_tkinter()
    global status_file_path
    status_file_path = ask_where_are_the_status_file()
    # Add bot token
    global TOKEN
    TOKEN = input("enter token: ")
    # Added all the essential command handlers
    updater = Updater(TOKEN)
    dp = updater.dispatcher
    dp.add_handler(CommandHandler('start', startmessage))
    dp.add_handler(CommandHandler('status', send_status))
    updater.start_polling()
    updater.idle()



from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from wplay.utils import Logger
from wplay.utils.helpers import logs_path


#region LOGGER create
logger : Logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


async def chat(target):
    logger.info("Chatting with target")
    page, _ = await browser_config.configure_browser_and_load_whatsapp()

    if target is not None:
        try:
            await target_search.search_and_select_target(page, target)
        except Exception as e:
            print(e)
            await page.reload()
            await target_search.search_and_select_target_without_new_chat_button(page, target)
    else:
        await target_select.manual_select_target(page)

    print("\033[91m {}\033[00m".format("\nType '...' in a new line or alone in the message to change target person.\nType '#_FILE' to send Image/Video/Documentd etc.\n"))

    while True:
        message : list[str] = io.ask_user_for_message_breakline_mode()

        if '...' in message:
            message.remove('...')
            await io.send_message(page, message)
            target = input("\n\nNew Target Name: ")
            if target is not None:
                await target_search.search_and_select_target(page, target)
            else:
                await target_select.manual_select_target(page)
            message = io.ask_user_for_message_breakline_mode()

        #File Share:
        if '#_FILE' in message:
            message.remove('#_FILE')
            await io.send_file(page)

        await io.send_message(page, message)



import time
import random

from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from wplay.utils import Logger
from wplay.utils.helpers import logs_path


#region LOGGER create
logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


async def msgTimer(target):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        try:
            await target_search.search_and_select_target(page, target)
        except Exception as e:
            print(e)
            await page.reload()
            await target_search.search_and_select_target_without_new_chat_button(page, target)
    else:
        await target_select.manual_select_target(page)
    # Region INPUTS
    logger.info("Input message information for message timer")
    message_type_numbers : int = int(
        input("How many types of messages will you send? "))
    messages : list[str] = list()
    for _ in range(message_type_numbers):
        messages.append(io.ask_user_for_message_breakline_mode())
    number_of_messages : int = int(input("Enter the number of messages to send: "))
    minimumTimeInterval : int = int(
        input("Enter minimum interval number in seconds: "))
    maximumTimeInterval : int = int(
        input("Enter maximum interval number in seconds: "))
    # Endregion

    random.seed()
    for _ in range(number_of_messages):
        if not messages:
            break
        await io.send_message(page, messages[random.randrange(0, message_type_numbers)])
        if minimumTimeInterval != maximumTimeInterval:
            time.sleep(random.randrange(minimumTimeInterval, maximumTimeInterval))
        else:
            time.sleep(minimumTimeInterval)



from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils import browser_config
import time
import sys
import requests


async def changeAbout():
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    choice = input(
        "Choose 1 for changing About section every second and 2 for changing About in every 30 min : ")
    if choice not in ['1', '2']:
        print("Wrong choice")
        sys.exit()
    await page.waitForSelector(
        whatsapp_selectors_dict['profile_photo_element'],
        visible=True
    )
    await page.click(whatsapp_selectors_dict['profile_photo_element'])

    while True:
        await page.waitForSelector(whatsapp_selectors_dict['about_edit_button_element'])
        await page.click(whatsapp_selectors_dict['about_edit_button_element'])
        for _ in range(140):
            await page.keyboard.press('Backspace')
        await page.type(whatsapp_selectors_dict['about_text_area'], get_live_cases())
        await page.keyboard.press('Enter')
        if choice == '1':
            time.sleep(1)
        else:
            time.sleep(1800)


def get_live_cases():
    '''
    Gives total cases live in world for coronavirus(IN)
    '''
    URL = "https://thevirustracker.com/free-api?global=stats"
    data = requests.get(URL, headers={"User-Agent": "XY"}).json()

    live_cases = data['results'][0]['total_cases']

    return str("Total Coronavirus Cases: " + str(live_cases))



import signal
import psutil


def kill_child_processes(parent_pid, sig=signal.SIGTERM):
    try:
        parent = psutil.Process(parent_pid)
    except psutil.NoSuchProcess:
        return
    children = parent.children(recursive=True)
    print('Process Killed!')
    for process in children:
        process.send_signal(sig)



__author__ = 'Pooja Sahu, github: poojasahu10'

# region IMPORTS
from wplay.utils.helpers import whatsapp_selectors_dict
# endregion

# region FOR SCRIPTING
async def manual_select_target(page, hide_groups=False):
    __print_manual_selection_info()
    await __open_new_chat(page)
    target_focused_title = await __get_focused_target_title(page)
    await __wait_for_message_area(page)
    __print_selected_target_title(target_focused_title)
    return target_focused_title
# endregion

# region SELECT TARGET
def __print_manual_selection_info():
    print(f"You've to go to whatsapp web and select target manually")

def __print_selected_target_title(target_focused_title):
    print(f"You've selected the target named by: {target_focused_title}")

async def __open_new_chat(page):
    await page.waitForSelector(
        whatsapp_selectors_dict['new_chat_button'],
        visible=True,
        timeout=0
    )

async def __get_focused_target_title(page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['target_focused_title'], visible=True,
        timeout=0)
        target_focused_title = await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["target_focused_title"]}").getAttribute("title")')
    except Exception as e:
        print(f'No target selected! Error: {str(e)}')
        exit()
    return target_focused_title

async def __wait_for_message_area(page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['message_area'], timeout=0)
    except Exception as e:
        print(f"You don't belong this group anymore! Error: {str(e)}")
# endregion


# region Imports
import os
import stat
import shutil
from pathlib import Path

from whaaaaat import Separator, prompt
# https://github.com/pytransitions/transitions#the-non-quickstart
from transitions import Machine, State

from wplay.utils.helpers import user_data_folder_path
from wplay.utils.helpers import menu_style
# endregion


class CliWhatsappPlay(object):

    def __init__(self):
        self.data_filenames = None  # type : list
        self.questions_menu = None  # type : list
        self.question_overwrite = None  # type : list
        self.answers_menu = None  # type : dict
        self.username = None  # type : str
        self.save_session = False  # type : bool
        self.user_options = {
            'restore': 'Restore a session',
            'save': 'Create a new session',
            'continue': 'Continue without saving',
            'delete': 'Delete a session',
            'exit': 'Exit'
        }    # type : dict

    def reset_fields(self):
        self.data_filenames = None  # type : list
        self.questions_menu = None  # type : list
        self.question_overwrite = None  # type : list
        self.answers_menu = None  # type : dict
        self.username = None  # type : str
        self.save_session = False  # type : bool


    def create_user_data_folder(self):
        Path(user_data_folder_path).mkdir(parents=True, exist_ok=True)

    def get_user_data_filenames(self):
        self.data_filenames = [file.stem for file in user_data_folder_path.glob('*')]

    def prepare_questions(self):
        self.questions_menu = [
            {
                'type': 'rawlist',
                'name': 'user_options',
                'message': '***Session Manager***:',
                'choices': [
                    Separator(),
                    self.user_options['restore'],
                    self.user_options['save'],
                    self.user_options['continue'],
                    Separator(),
                    self.user_options['delete'],
                    self.user_options['exit'],
                    Separator()
                ]
            },
            {
                'type': 'rawlist',
                'name': 'restore',
                'message': 'Select a session to try to restore:',
                'choices': [*[session for session in self.data_filenames], '<---Go-back---'],
                'when': lambda answers: answers['user_options'] == self.user_options['restore']
            },
            {
                'type': 'input',
                'name': 'save',
                'message': 'Write your first name or username to save:',
                'when': lambda answers: answers['user_options'] == self.user_options['save']
            },
            {
                'type': 'checkbox',
                'name': 'delete',
                'message': 'Mark the sessions you want to delete:',
                'choices': list(map(lambda e: {'name': e}, self.data_filenames)),
                'when': lambda answers: answers['user_options'] == self.user_options['delete']
            }
        ]

        self.question_overwrite = [
            {
                'type': 'confirm',
                'name': 'overwrite_data',
                'message': 'There is already a session with that name, overwrite it?',
                'default': True
            }
        ]
    
    def get_answer_menu(self):
        self.answers_menu = prompt(self.questions_menu, style=menu_style)  

    def verify_answers(self):

        # Handle when person choose 'Restore a session'
        if self.answers_menu['user_options'] == self.user_options['restore']:
            if self.answers_menu['restore'] == '<---Go-back---':
                return False
            else:
                self.username = self.answers_menu['restore']
                self.save_session = True
                return True

        # Handle when person choose 'Create a new session'
        elif self.answers_menu['user_options'] == self.user_options['save']:
            self.username = self.answers_menu['save']
            self.save_session = True
            return self.__verify_if_session_file_exists()
            

        # Handle when person choose 'Continue without saving'
        elif self.answers_menu['user_options'] == self.user_options['continue']:
            self.username, self.save_session = None, False
            return True

        # Handle when person choose 'Delete a session'
        elif self.answers_menu['user_options'] == self.user_options['delete']:
            if len(self.answers_menu['delete']) > 0:
                [self.__delete_session_data(user_data_folder_path / username) for username in self.answers_menu['delete']]
            return False

        # Handle when person choose 'Exit'
        elif self.answers_menu['user_options'] == self.user_options['exit']:
            exit()

    def __verify_if_session_file_exists(self):
        if self.username in self.data_filenames:
            answer_overwrite = prompt(self.question_overwrite, style=menu_style)
            if answer_overwrite['overwrite_data']:
                self.__delete_session_data(user_data_folder_path / self.username)
            return answer_overwrite['overwrite_data']
        return True


    def __delete_session_data(self, path):
        def handleError(func, path, exc_info):
            print('Handling Error for file ', path)
            if not os.access(path, os.W_OK):
                print('Trying to change permission!')
                os.chmod(path, stat.S_IWUSR)
                shutil.rmtree(path, ignore_errors=True)

        shutil.rmtree(path, onerror=handleError)


states = [
    State(name='start'),
    State(name='create_user_data_folder', on_enter='create_user_data_folder'),
    State(name='get_user_data_filenames', on_enter='get_user_data_filenames'),
    State(name='prepare_questions', on_enter='prepare_questions'),
    State(name='get_answer_menu', on_enter='get_answer_menu'),
    State(name='verify_answers', on_enter='verify_answers')
]

transitions = [
    {"trigger": 'start', 'source': '*', 'dest': 'start'},
    { 'trigger': 'create_user_data_folder', 'source': 'start', 'dest': 'create_user_data_folder'},
    { 'trigger': 'get_user_data_filenames', 'source': 'create_user_data_folder', 'dest': 'get_user_data_filenames'},
    { 'trigger': 'prepare_questions', 'source': 'get_user_data_filenames', 'dest': 'prepare_questions'},
    { 'trigger': 'get_answer_menu', 'source': 'prepare_questions', 'dest': 'get_answer_menu'},
    { 'trigger': 'verify_answers', 'source': 'get_answer_menu', 'dest': 'verify_answers'},
]

def session_manager():
    done = False
    obj = CliWhatsappPlay()
    while(not done):
        if(obj.questions_menu != None): obj.reset_fields()
        machine = Machine(obj, states, transitions=transitions, initial='start')
        obj.create_user_data_folder()
        obj.get_user_data_filenames()
        obj.prepare_questions()
        obj.get_answer_menu()
        done = obj.verify_answers()

    return obj.username, obj.save_session



__author__ = 'Alexandre Calil Martins Fonseca, github: xandao6'


# region TUTORIAL
'''
Go to region 'FOR SCRIPTING' and use the methods in your script!

EXAMPLE OF USAGE:
from wplay.pyppeteerUtils import pyppeteerConfig as pypConfig
from wplay.pyppeteerUtils import pyppeteerSearch as pypSearch

async def my_script(target):
    pages, browser = wait pyp.configure_browser_and_load_whatsapp(pypConfig.websites['whatsapp'])
    await pypSearch.search_for_target_and_get_ready_for_conversation(pages[0], target)

    message = pypSearch.ask_user_for_message_breakline_mode()
    await pypSearch.send_message(pages[0], message)

    message2 = pypSearch.ask_user_for_message()
    await pypSearch.send_message(pages[0], message2)

'''
# endregion


# region IMPORTS
from pyppeteer import launch
from wplay.utils.session_manager import session_manager
from wplay.utils.helpers import websites, user_data_folder_path
from wplay.utils import Logger
from wplay.utils.helpers import logs_path
# endregion


# region FOR SCRIPTING
async def configure_browser_and_load_whatsapp():
    __patch_pyppeteer()
    username, save_session = session_manager()
    browser = await __config_browser(username, save_session)
    pages : list[int] = await __config_pages(browser)
    return pages[0], browser
# endregion


#region LOGGER create
logger : Logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


# region PYPPETEER PATCH
# https://github.com/miyakogi/pyppeteer/pull/160
# HACK: We need this until this PR is accepted. Solves the bug bellow.
# BUG:(Pyppeteer) The communication with Chromium are disconnected after 20s.
def __patch_pyppeteer():
    from typing import Any
    from pyppeteer import connection, launcher
    import websockets.client
    logger.debug("Using Pyppeteer for connecting to Chromium")

    class PatchedConnection(connection.Connection):  # type: ignore
        def __init__(self, *args: Any, **kwargs: Any) -> None:
            super().__init__(*args, **kwargs)
            self._ws = websockets.client.connect(
                self._url,
                loop=self._loop,
                max_size=None,  # type: ignore
                ping_interval=None,  # type: ignore
                ping_timeout=None,  # type: ignore
            )

    connection.Connection = PatchedConnection
    launcher.Connection = PatchedConnection
# endregion


# region PYPPETEER CONFIGURATION
async def __config_browser(username = None, save_session = False):
    if username is not None and username != '' and save_session:
        logger.info('Configuring Browser')
        return await launch(
            headless = False,
            autoClose = False,
            userDataDir = user_data_folder_path / username
        )
    else:
        return await launch(headless = False, autoClose = False)


async def __config_pages(browser):
    logger.info('Opening Browser')
    pages : list[int] = await __get_pages(browser)
    await __set_user_agent(pages[0])
    # await __set_view_port(pages[0])
    await __open_website(pages[0], websites['whatsapp'])
    return pages


async def __open_new_page(browser):
    await browser.newPage()


async def __get_pages(browser):
    return await browser.pages()


async def __set_user_agent(page):
    await page.setUserAgent('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36')


async def __set_view_port(page):
    await page.setViewport({'width': 1280, 'height': 800})


async def __open_website(page, website):
    await page.bringToFront()
    await page.goto(website, waitUntil='networkidle2', timeout=0)


def __exit_if_wrong_url(page, browser, url_to_check):
    if not page.url == url_to_check:
        logger.error('Exit due to Wrong URL!')
        print("Wrong URL!")
        browser.close()
        exit()
        return
# endregion


# region CODE THAT MIGHT BE USEFUL SOMEDAY
'''
# FIX:
# To load websites faster
async def intercept(request, page_one, page_two):
    await page_one.setRequestInterception(True)
    await page_two.setRequestInterception(True)
    if any(request.resourceType == _ for _ in ('stylesheet', 'image', 'font', 'media')):
        await request.abort()
    else:
        await request.continue_()
    page.on('request', lambda req: asyncio.ensure_future(intercept(req)))
'''
# endregion



# region IMPORTS
from pathlib import Path
from whaaaaat import style_from_dict, Token
# endregion

# region WEBSITES
websites = {'whatsapp': 'https://web.whatsapp.com/'}
# endregion

# region SELECTOR
whatsapp_selectors_dict = {
    'login_area':'#app > div > div > div.landing-header',

    'new_chat_button': '#side > header div[role="button"] span[data-icon="chat"]',
    'search_contact_input_new_chat': '#app > div > div > div > div > span > div > span > div > div > div > label > div > div',
    'contact_list_elements_filtered_new_chat': '#app > div > div span > div > span > div div > div div > div div > span > span[title][dir]',
    'group_list_elements_filtered_new_chat': '#app > div > div span > div > span > div div > div div > div div > span[title][dir]',

    'search_contact_input':'#side > div > div > label > div > div',
    'chat_list_elements_filtered':'#pane-side > div > div > div > div > div > div > div > div > div > span > span[title][dir]',

    'target_focused_title': '#main > header div > div > span[title]',
    'message_area': '#main > footer div.selectable-text[contenteditable]',
    'last_seen':'#main > header > div > div > span[title]',
    'target_chat_header': '#main > header',
    'contact_info_page_elements': '#app > div > div > div:nth-child(2) > div:last-of-type > span > div > span > div > div > div:first-child > div',
    'contact_info_page_group_element_heading': '#app > div > div > div:nth-child(2) > div:last-of-type > '
                                    'span > div > span > div > div > div:first-child > div:nth-child(5)>div>div>div>div:first-child>span',
    'contact_info_page_group_elements': '#app > div > div > div:nth-child(2) > div:last-of-type > '
                                    'span > div > span > div > div > div:first-child > div:nth-child(5)>div:nth-child(2)>div>div',
    'contact_info_page_close_button': '#app > div > div > div:nth-child(2) > div:last-of-type > span > div > span > div > header > div > div > button',
    'chat_or_message_search': '#side > div:nth-child(3) > div > label > div > div:last-child',
    'chats_groups_messages_elements': '#side > div:last-child > div > div > div > div',
    'contact_element': 'span > span > span[class^="matched-text"]',
    'group_element': 'div:last-child > div:first-child > div:first-child > div > span > span[class^="matched-text"]',
    'attach_file' : '#main > header > div > div > div:nth-child(2) > div',
    'choose_file' : '#main > header > div > div > div > span > div > div > ul > li:nth-child(3) > button',
    'send_file' : '#app > div > div > div > div > span > div > span > div > div > div > span > div > div > span',
    'profile_photo_element': '#side > header > div > div > img',
    'about_edit_button_element': '#app > div > div > div > div > span > div > div > div > div:nth-child(4) > div > div > span > div > span',
    'about_text_area': '#app > div > div > div > div > span > div > div > div > div:nth-child(4) > div > div > div > div'
}
# endregion

# region PATHS
data_folder_path = Path.home() / 'wplay'
logs_path = Path.home() / 'wplay' / 'logs'
user_data_folder_path = Path.home() / 'wplay' / '.userData'
# endregion

# region MENU STYLES
menu_style = style_from_dict({
    Token.Separator: '#6C6C6C',
    Token.QuestionMark: '#FF9D00 bold',
    Token.Selected: '#5F819D',
    Token.Pointer: '#FF9D00 bold',
    Token.Instruction: '',  # default
    Token.Answer: '#5F819D bold',
    Token.Question: '',
})
# endregion



# the code is not finalised.


def locationfinder(name):

    """
    # to find location by ip address
    print('Get you ipinfo token from https://ipinfo.io/account')
    ip_address = '*'
    token = str(input("Enter your ipinfo token: "))
    ip_string = 'curl ipinfo.io/'+ip_address+'?token='+token+''
    os.system(ip_string)
    """



__author__ = 'Alexandre Calil Martins Fonseca, github: xandao6'


# region TUTORIAL
'''
Go to region 'FOR SCRIPTING' and use the methods in your script!

EXAMPLE OF USAGE:
from wplay.pyppeteerUtils import pyppeteerConfig as pypConfig
from wplay.pyppeteerUtils import pyppeteerSearch as pypSearch

async def my_script(target):
    pages, browser = wait pyp.configure_browser_and_load_whatsapp(pypConfig.websites['whatsapp'])
    await pypSearch.search_for_target_and_get_ready_for_conversation(pages[0], target)

    message = pypSearch.ask_user_for_message_breakline_mode()
    await pypSearch.send_message(pages[0], message)

    message2 = pypSearch.ask_user_for_message()
    await pypSearch.send_message(pages[0], message2)
'''
# endregion


# region IMPORTS
from wplay.utils.helpers import whatsapp_selectors_dict
from wplay.utils import Logger
from wplay.utils.helpers import logs_path
from pyppeteer.errors import ElementHandleError
# endregion


# region FOR SCRIPTING
async def search_and_select_target(page, target, hide_groups=False):
    try: 
        if int(target):
            await __open_website(page, f'https://wa.me/{target}')
    except:
        pass    
    await __open_new_chat(page)
    await __type_in_new_chat_search_bar(page, target)
    contact_list_elements_unchecked = await __get_contacts_elements_filtered(page, target)
    group_list_elements_unchecked = await __get_groups_elements_filtered(page, target, hide_groups)
    contact_titles_unchecked = await __get_contacts_titles_from_elements_unchecked(page, contact_list_elements_unchecked)
    group_titles_unchecked = await __get_groups_titles_from_elements_unchecked(page, group_list_elements_unchecked)
    contact_list_unchecked = __zip_contact_titles_and_elements_unchecked(
        contact_titles_unchecked, contact_list_elements_unchecked)
    group_list_unchecked = __zip_group_titles_and_elements_unchecked(
        group_titles_unchecked, group_list_elements_unchecked)
    contact_tuple = __check_contact_list(target, contact_list_unchecked)
    group_tuple = __check_group_list(target, group_list_unchecked)
    target_tuple = __get_target_tuple(contact_tuple, group_tuple)
    __print_target_tuple(target_tuple)
    target_index_choosed = __ask_user_to_choose_the_filtered_target(target_tuple)
    choosed_target = __get_choosed_target(target_tuple, target_index_choosed)
    await __navigate_to_target(page, choosed_target)
    target_focused_title = await __get_focused_target_title(page, target)
    if any(choosed_target[0] in i for i in contact_tuple):
        complete_target_info = await __get_complete_info_on_target(page)
        __print_complete_target_info(complete_target_info)
        await __close_contact_info_page(page)
    else:
        __print_selected_target_title(target_focused_title)
    __check_target_focused_title(page, target, target_focused_title)
    await __wait_for_message_area(page)
    return target_focused_title


async def search_and_select_target_without_new_chat_button(page,target, hide_groups=False):
    await __type_in_chat_or_message_search(page,target)
    chats_messages_groups_elements_list = await __get_chats_messages_groups_elements(page)
    contact_name_index_tuple_list = await __get_contacts_matched_with_query(chats_messages_groups_elements_list)
    group_name_index_tuple_list = await __get_groups_matched_with_query(chats_messages_groups_elements_list,hide_groups)
    await __get_number_of_all_contacts(page, contact_name_index_tuple_list, chats_messages_groups_elements_list)
    target_tuple = (contact_name_index_tuple_list,group_name_index_tuple_list)
    __print_target_tuple(target_tuple)
    target_index_chosen = __ask_user_to_choose_the_filtered_target(target_tuple)

    #chosen_target will be a tuple (a,b) such that a is the name of the target and b is the
    #index of that element in chats_messages_groups_elements_list

    chosen_target = __get_choosed_target(target_tuple, target_index_chosen)
    await __open_selected_chat(chosen_target[1],chats_messages_groups_elements_list)
    target_name = chosen_target[0]
    if any(chosen_target[0] in i for i in contact_name_index_tuple_list):
        complete_target_info = await __get_complete_info_on_target(page)
        __print_complete_target_info(complete_target_info)
        await __close_contact_info_page(page)
    else:
        __print_selected_target_title(target_name)
    await __wait_for_message_area(page)
    return target_name


# endregion


#region LOGGER create
logger : Logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


# region SEARCH AND SELECT TARGET
async def __get_number_of_all_contacts(page,contact_name_index_tuple_list, chats_messages_groups_elements_list):
    try:
        for index,contact_name_index_tuple in enumerate(contact_name_index_tuple_list):
            await chats_messages_groups_elements_list[contact_name_index_tuple[1]].click()
            await __open_target_chat_info_page(page)
            contact_page_elements = await __get_contact_page_elements(page)
            complete_target_info = {}
            await __get_contact_about_and_phone(contact_page_elements[3], complete_target_info)
            contact_name_index_tuple_list[index] = (contact_name_index_tuple[0] + " : " + complete_target_info['Mobile'],contact_name_index_tuple[1])
            await __close_contact_info_page(page)
    except Exception as e:
        print(e)

async def __type_in_chat_or_message_search(page,target):
    try:
        print(f'Looking for: {target}')
        await page.waitForSelector(
            whatsapp_selectors_dict['chat_or_message_search'],
            visible=True,
            timeout=0
        )
        await page.waitFor(500)
        await page.type(whatsapp_selectors_dict['chat_or_message_search'], target)
        await page.waitFor(3000)
    except Exception as e:
        print(e)


async def __get_chats_messages_groups_elements(page):
    chats_messages_groups_elements_list = []  # type : list[int]
    try:
        chats_messages_groups_elements_list = await page.querySelectorAll\
            (whatsapp_selectors_dict['chats_groups_messages_elements'])
        return chats_messages_groups_elements_list
    except Exception as e:
        print(e)
        exit()


async def __get_contacts_matched_with_query(chats_groups_messages_elements_list):
    contacts_to_choose_from = []  # type : list[str , int]
    get_contact_node_title_function = 'node => node.parentNode.getAttribute("title")'
    for idx, element in enumerate(chats_groups_messages_elements_list):
        try:
            contact_name = await element.querySelectorEval(whatsapp_selectors_dict['contact_element'],get_contact_node_title_function)
            if contact_name is not None:
                contacts_to_choose_from.append((contact_name,idx))
        except ElementHandleError:
            # if it is not a contact element, move to the next one
            continue
        except Exception as e:
            print(e)

    return contacts_to_choose_from


async def __get_groups_matched_with_query(chats_groups_messages_elements_list,hide_groups):
    groups_to_choose_from = []

    if hide_groups:
        return groups_to_choose_from

    get_group_node_title_function = 'node => node.parentNode.getAttribute("title")'
    for idx, element in enumerate(chats_groups_messages_elements_list):
        try:
            group_name = await element.querySelectorEval(whatsapp_selectors_dict['group_element'],
                                                         get_group_node_title_function)
            groups_to_choose_from.append((group_name,idx))
        except ElementHandleError:
            # if it is not a contact element, move to the next one
            continue
        except Exception as e:
            print(e)

    return groups_to_choose_from


async def __open_selected_chat(target_index,chats_messages_groups_elements_list):
    try:
        await chats_messages_groups_elements_list[target_index].click()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        exit()


async def __get_complete_info_on_target(page):
    contact_page_elements = []
    try:
        await __open_target_chat_info_page(page)
        contact_page_elements = await __get_contact_page_elements(page)
        complete_target_info = {}
        await __get_contact_name_info(contact_page_elements[0], complete_target_info)
        await __get_contact_about_and_phone(contact_page_elements[3], complete_target_info)
        await __get_contact_groups_common_with_target(complete_target_info, page)
        return complete_target_info
    except Exception as e:
        print(e)


async def __open_target_chat_info_page(page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['target_chat_header'],
            visible=True,
            timeout=3000
        )
        await page.click(whatsapp_selectors_dict['target_chat_header'])
    except Exception as e:
        print(e)


async def __get_contact_page_elements(page):
    contact_page_elements = []
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_elements'],
            visible=True,
            timeout=8000
        )
        contact_page_elements = await page.querySelectorAll(whatsapp_selectors_dict['contact_info_page_elements'])
        return contact_page_elements
    except Exception as e:
        print(e)



async def __get_contact_name_info(contact_name_element,complete_target_info):
    try:
        complete_target_info['Name'] = await contact_name_element.querySelectorEval('span > span', 'element => element.innerText')
        complete_target_info['Last_seen'] = await contact_name_element.querySelectorEval('div > span:last-of-type > div > span', 'element => element.getAttribute("title")')
    except:
        print(f'last seen not available')


async def __get_contact_about_and_phone(contact_name_element, complete_target_info):
    try:
        complete_target_info['About'] = await contact_name_element.querySelectorEval('div:nth-child(2) > div > div > span > span', 'element => element.getAttribute("title")')
        complete_target_info['Mobile'] = await contact_name_element.querySelectorEval('div:last-of-type > div > div > span > span', 'element => element.innerText')
    except Exception as e:
        print(e)


async def __get_contact_groups_common_with_target(complete_target_info,page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_group_element_heading'],
            visible= True,
            timeout=3000
        )

        if (await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["contact_info_page_group_element_heading"]}").innerText'))\
               == "Groups in common":
            group_elements = await page.querySelectorAll(whatsapp_selectors_dict['contact_info_page_group_elements'])
            complete_target_info['Groups'] = [await ele.querySelectorEval('div>div>div:nth-child(2)>div:first-child>div>div>span', 'e => e.getAttribute("title")') for ele in group_elements]
        else:
            complete_target_info['Groups'] = []
    except:
        complete_target_info['Groups'] = []
        print(f'No groups in common')


async def __close_contact_info_page(page):
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_info_page_close_button'],
            visible = True,
            timeout = 5000
        )
        await page.click(whatsapp_selectors_dict['contact_info_page_close_button'])
    except Exception as e:
        print(e)


def __print_complete_target_info(complete_target_info):
    for key in complete_target_info.keys():
        if key == "Groups":
            print("Groups:")
            print(*complete_target_info[key], sep=",")
        else:
            print(f'{key}: {complete_target_info[key]} ')


async def __open_new_chat(page):
    await page.waitForSelector(
        whatsapp_selectors_dict['new_chat_button'],
        visible=True,
        timeout=0
    )
    await page.waitFor(500)
    await page.click(whatsapp_selectors_dict['new_chat_button'])


async def __type_in_new_chat_search_bar(page, target):
    print(f'Looking for: {target}')
    logger.info('Searching Target')
    await page.waitForSelector(
        whatsapp_selectors_dict['search_contact_input_new_chat'],
        visible=True
    )
    await page.type(whatsapp_selectors_dict['search_contact_input_new_chat'], target)
    await page.waitFor(3000)


async def __get_contacts_elements_filtered(page, target):
    contact_list_elements_unchecked = list()
    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['contact_list_elements_filtered_new_chat'],
            visible=True,
            timeout=3000
        )

        contact_list_elements_unchecked = await page.querySelectorAll(
            whatsapp_selectors_dict['contact_list_elements_filtered_new_chat']
        )
    except:
        print(f'No contact named by "{target}"!')
        logger.info('Target not found')
    return contact_list_elements_unchecked


async def __get_groups_elements_filtered(page, target, hide_groups=False):
    group_list_elements_unchecked = list()

    if hide_groups:
        return group_list_elements_unchecked

    try:
        await page.waitForSelector(
            whatsapp_selectors_dict['group_list_elements_filtered_new_chat'],
            visible=True,
            timeout=3000
        )

        group_list_elements_unchecked = await page.querySelectorAll(
            whatsapp_selectors_dict['group_list_elements_filtered_new_chat']
        )
    except:
        print(f'No group named by "{target}"!')
        logger.info('Target not found in groups')
    return group_list_elements_unchecked


async def __get_contacts_titles_from_elements_unchecked(page, contact_list_elements_unchecked):
    contact_titles_unchecked = []
    for i in range(len(contact_list_elements_unchecked)):
        contact_titles_unchecked\
            .append(await page.evaluate(f'document.querySelectorAll("{whatsapp_selectors_dict["contact_list_elements_filtered_new_chat"]}")[{i}].getAttribute("title")'))
    return contact_titles_unchecked


async def __get_groups_titles_from_elements_unchecked(page, group_list_elements_unchecked):
    group_titles_unchecked = []
    for i in range(len(group_list_elements_unchecked)):
        group_titles_unchecked.append(await page.evaluate(f'document.querySelectorAll("{whatsapp_selectors_dict["group_list_elements_filtered_new_chat"]}")[{i}].getAttribute("title")'))
    return group_titles_unchecked


# contact_list_unchecked is a zip (list of tuples) of contact_titles and
# contact elements, unchecked.
def __zip_contact_titles_and_elements_unchecked(contact_titles_unchecked, contact_list_elements_unchecked):
    contact_list_unchecked = list(zip(contact_titles_unchecked, contact_list_elements_unchecked))
    return contact_list_unchecked


def __zip_group_titles_and_elements_unchecked(group_titles_unchecked, group_list_elements_unchecked):
    group_list_unchecked = list(zip(group_titles_unchecked, group_list_elements_unchecked))
    return group_list_unchecked


# __checking_contact_list verify if target is in title, if not we pop from list
def __check_contact_list(target, contact_list_unchecked):
    i = 0
    while i < len(contact_list_unchecked):
        if len(contact_list_unchecked) <= 0:
            break

        # we can add more verifications if we are getting false-positive contacts
        if contact_list_unchecked[i][0].lower().find(target.lower()) == -1:
            try:
                contact_list_unchecked.pop(i)
            except Exception as e:
                print(f'Error: {str(e)}')
            i -= 1
        i += 1

    contact_tuple = tuple(contact_list_unchecked)
    return contact_tuple


def __check_group_list(target, group_list_unchecked):
    i = 0
    while i < len(group_list_unchecked):
        if len(group_list_unchecked) <= 0:
            break

        # we can add more verifications if we are getting false-positive groups
        if group_list_unchecked[i][0].lower().find(target.lower()) == -1:
            try:
                group_list_unchecked.pop(i)
            except Exception as e:
                print(f'Error: {str(e)}')
            i -= 1
        i += 1

    group_tuple = tuple(group_list_unchecked)
    return group_tuple


# target_list is like that: (((0, 'a'), (1, 'b')), ((3, 'c'), (4, 'd'))),
# but instead numbers and letters we have titles and elements
# the first index is the contacts and the second is the groups
def __get_target_tuple(contact_tuple, group_tuple):
    target_tuple = (contact_tuple, group_tuple)
    return target_tuple


def __print_target_tuple(target_tuple):
    lenght_of_contacts_tuple = len(target_tuple[0])
    lenght_of_groups_tuple = len(target_tuple[1])

    for i in range(lenght_of_contacts_tuple):
        if lenght_of_contacts_tuple <= 0:
            break
        if i == 0:
            print("Contacts found:")
            logger.info('List of Targets')
        print(f'{i}: {target_tuple[0][i][0]}')

    for i in range(lenght_of_contacts_tuple, lenght_of_groups_tuple + lenght_of_contacts_tuple):
        if lenght_of_groups_tuple <= 0:
            break
        if i == lenght_of_contacts_tuple:
            print("Groups found:")
            logger.info('List of Target in groups')
        print(f'{i}: {target_tuple[1][i-lenght_of_contacts_tuple][0]}')


def __ask_user_to_choose_the_filtered_target(target_tuple):
    if len(target_tuple[0] + target_tuple[1]) > 0:
        logger.info('Input Target Number')
        target_index_choosed = int(
            input('Enter the number of the target you wish to choose: '))
    return target_index_choosed


def __get_choosed_target(target_tuple, target_index_choosed):
    lenght_of_contacts_tuple = len(target_tuple[0])
    if target_index_choosed is None:
        exit()

    try:
        if target_index_choosed < lenght_of_contacts_tuple:
            choosed_target = target_tuple[0][target_index_choosed]
        elif target_index_choosed >= lenght_of_contacts_tuple:
            choosed_target = target_tuple[1][target_index_choosed - lenght_of_contacts_tuple]
        else:
            print("This target doesn't exist!")
            logger.error('Invalid Target')
            exit()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        logger.error('Invalid Target')
        exit()
    return choosed_target


async def __navigate_to_target(page, choosed_target):
    try:
        await choosed_target[1].click()
    except Exception as e:
        print(f"This target doesn't exist! Error: {str(e)}")
        logger.error('Invalid Target')
        exit()


async def __get_focused_target_title(page, target):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['target_focused_title'])
        target_focused_title = await page.evaluate(f'document.querySelector("{whatsapp_selectors_dict["target_focused_title"]}").getAttribute("title")')
    except Exception as e:
        print(f'No target selected! Error: {str(e)}')
        logger.error('Target not selected from list')
        exit()
    return target_focused_title


def __print_selected_target_title(target_focused_title):
    print(f"You've selected the target named by: {target_focused_title}")
    logger.info('Selected Target')


def __check_target_focused_title(page, target, target_focused_title):
    if target_focused_title.lower().find(target.lower()) == -1:
        print(f"You're focused in the wrong target, {target_focused_title}")
        must_continue = str(input("Do you want to continue (yes/no)? "))
        accepted_yes = {'yes', 'y'}
        if must_continue.lower() in accepted_yes:
            pass
        else:
            exit()


async def __wait_for_message_area(page):
    try:
        await page.waitForSelector(whatsapp_selectors_dict['message_area'])
    except Exception as e:
        print(f"You don't belong this group anymore! Error: {str(e)}")
# endregion



'''
This code is copied by https://github.com/EliteAndroidApps/WhatsApp-GD-Extractor.


EliteAndroidApps/WhatsApp-GD-Extractor is licensed under the
GNU General Public License v3.0
'''


from configparser import ConfigParser
import json
import os
import re
import requests


def getGoogleAccountTokenFromAuth():
    payload : dict = {'Email': gmail, 'Passwd': passw, 'app': client_pkg,
               'client_sig': client_sig, 'parentAndroidId': devid}
    request = requests.post(
        'https://android.clients.google.com/auth', data=payload)
    token = re.search('Token=(.*?)\n', request.text)
    if token:
        return token.group(1)
    else:
        quit(request.text)


def getGoogleDriveToken(token):
    payload : dict = {
        'Token': token,
        'app': pkg,
        'client_sig': sig,
        'device': devid,
        'google_play_services_version': client_ver,
        'service': 'oauth2:https://www.googleapis.com/auth/drive.appdata https://www.googleapis.com/auth/drive.file',
        'has_permission': '1'
    }
    request = requests.post(
        'https://android.clients.google.com/auth', data=payload)
    token = re.search('Auth=(.*?)\n', request.text)
    if token:
        return token.group(1)
    else:
        quit(request.text)


def rawGoogleDriveRequest(bearer, url):
    headers : dict = {'Authorization': 'Bearer ' + bearer}
    request : dict = requests.get(url, headers=headers)
    return request.text


def downloadFileGoogleDrive(bearer, url, local):
    if not os.path.exists(os.path.dirname(local)):
        os.makedirs(os.path.dirname(local))
    if os.path.isfile(local):
        os.remove(local)
    headers : dict = {'Authorization': 'Bearer ' + bearer}
    request = requests.get(url, headers=headers, stream=True)
    request.raw.decode_content = True
    if request.status_code == 200:
        with open(local, 'wb') as asset:
            for chunk in request.iter_content(1024):
                asset.write(chunk)
    print('Downloaded: "' + local + '".')


def gDriveFileMap():
    global bearer
    data = rawGoogleDriveRequest(
        bearer, 'https://www.googleapis.com/drive/v2/files')
    jres = json.loads(data)
    backups = [] #  type : list
    for result in jres['items']:
        if result['title'] == 'gdrive_file_map':
            backups.append((result['description'],
                            rawGoogleDriveRequest(
                                bearer,
                                result['downloadUrl'])))
    if len(backups) == 0:
        quit('Unable to locate google drive file map for: ' + pkg)
    return backups


def getConfigs():
    global gmail, passw, devid, pkg, sig, client_pkg, client_sig, client_ver
    config = ConfigParser()
    try:
        config.read('settings.cfg')
        gmail = input("enter WhatsApp backup Email: ")  # type : str
        passw = input("enter password: ")  # type : str
        devid = config.get('auth', 'devid')
        pkg = config.get('app', 'pkg')
        sig = config.get('app', 'sig')
        client_pkg = config.get('client', 'pkg')
        client_sig = config.get('client', 'sig')
        client_ver = config.get('client', 'ver')
    except(ConfigParser.NoSectionError, ConfigParser.NoOptionError):
        quit('The "settings.cfg" file is missing or corrupt!')


def jsonPrint(data):
    print(json.dumps(json.loads(data), indent=4, sort_keys=True))


def localFileLog(md5):
    logfile = 'logs' / 'files.log'  # type : str
    if not os.path.exists(os.path.dirname(logfile)):
        os.makedirs(os.path.dirname(logfile))
    with open(logfile, 'a') as log:
        log.write(md5 + '\n')


def localFileList():
    logfile = 'logs' / 'files.log'  # type : str
    if os.path.isfile(logfile):
        flist = open(logfile, 'r')
        return [line.split('\n') for line in flist.readlines()]
    else:
        open(logfile, 'w')
        return localFileList()


def createSettingsFile():
    with open('settings.cfg', 'w') as cfg:
        cfg.write('[auth]\ngmail = alias@gmail.com\npassw = yourpassword\ndevid = 0000000000000000\n\n[app]\npkg = com.whatsapp\nsig = 38a0f7d505fe18fec64fbf343ecaaaf310dbd799\n\n[client]\npkg = com.google.android.gms\nsig = 38918a453d07199354f8b19af05ec6562ced5788\nver = 9877000')


def getSingleFile(data, asset):
    data = json.loads(data)
    for entries in data:
        if entries['f'] == asset:
            return entries['f'], entries['m'], entries['r'], entries['s']


def getMultipleFiles(data, folder):
    files = localFileList()
    data = json.loads(data)
    for entries in data:
        if any(entries['m'] in lists for lists in files) is \
                False or 'database' in entries['f'].lower():
            local = folder + os.path.sep + entries['f'].replace("/", os.path.sep)
            if os.path.isfile(local) and 'database' not in local.lower():
                quit('Skipped: "' + local + '".')
            else:
                downloadFileGoogleDrive(
                    bearer,
                    'https://www.googleapis.com/drive/v2/files/' + entries['r'] + '?alt=media',
                    local)
                localFileLog(entries['m'])


def runMain(mode, asset, bID):
    global bearer
    if not os.path.isfile('settings.cfg'):
        createSettingsFile()
    getConfigs()
    bearer = getGoogleDriveToken(getGoogleAccountTokenFromAuth())
    drives = gDriveFileMap()
    if mode == 'info':
        for i, drive in enumerate(drives):
            if len(drives) > 1:
                print("Backup: " + str(i))
            jsonPrint(drive[0])
    elif mode == 'list':
        for i, drive in enumerate(drives):
            if len(drives) > 1:
                print("Backup: " + str(i))
            jsonPrint(drive[1])
    elif mode == 'pull':
        try:
            drive = drives[bID]
        except IndexError:
            quit("Invalid backup ID: " + str(bID))
        target = getSingleFile(drive[1], asset)
        try:
            f = target[0]
            m = target[1]
            r = target[2]
        except TypeError:
            quit('Unable to locate: "' + asset + '".')
        local = 'WhatsApp'+ os.path.sep + f.replace("/", os.path.sep)
        if os.path.isfile(local) and 'database' not in local.lower():
            quit('Skipped: "' + local + '".')
        else:
            downloadFileGoogleDrive(
                bearer,
                'https://www.googleapis.com/drive/v2/files/' + r + '?alt=media',
                local)
            localFileLog(m)
    elif mode == 'sync':
        for i, drive in enumerate(drives):
            folder = 'WhatsApp'
            if len(drives) > 1:
                print('Backup: ' + str(i))
                folder = 'WhatsApp-' + str(i)
            getMultipleFiles(drive[1], folder)


'''
def main():
    args = len(sys.argv)
    if  args < 2 or str(sys.argv[1]) == '-help' or str(sys.argv[1]) == 'help':
        print('\nUsage: '+str(sys.argv[0])+' -help|-vers|-info|-list|-sync|-pull file [backupID]\n\nExamples:\n')
        print('python '+str(sys.argv[0])+' -help (this help screen)')
        print('python '+str(sys.argv[0])+' -vers (version information)')
        print('python '+str(sys.argv[0])+' -info (google drive app settings)')
        print('python '+str(sys.argv[0])+' -list (list all availabe files)')
        print('python '+str(sys.argv[0])+' -sync (sync all files locally)')
        print('python '+str(sys.argv[0])+' -pull "Databases/msgstore.db.crypt12" [backupID] (download)\n')
    elif str(sys.argv[1]) == '-info' or str(sys.argv[1]) == 'info':
        runMain('info', 'settings', 0)
    elif str(sys.argv[1]) == '-list' or str(sys.argv[1]) == 'list':
        runMain('list', 'all', 0)
    elif str(sys.argv[1]) == '-sync' or str(sys.argv[1]) == 'sync':
        runMain('sync', 'all', 0)
    elif str(sys.argv[1]) == '-vers' or str(sys.argv[1]) == 'vers':
        print('\nWhatsAppGDExtract Version 1.1 Copyright (C) 2016 by TripCode\n')
    elif args < 3:
        quit('\nUsage: python '+str(sys.argv[0])+' -help|-vers|-info|-list|-sync|-pull file [backupID]\n')
    elif str(sys.argv[1]) == '-pull' or str(sys.argv[1]) == 'pull':
        try:
            bID = int(sys.argv[3])
        except (IndexError, ValueError):
            bID = 0
        runMain('pull', str(sys.argv[2]), bID)
    else:
        quit('\nUsage: python '+str(sys.argv[0])+' -help|-vers|-info|-list|-sync|-pull file [backupID]\n')

if __name__ == "__main__":
    main()
'''



# region IMPORTS
import logging
from pathlib import Path
from wplay.utils.helpers import logs_path
# endregion


# region for FORMAT
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
# endregion


# region for logger class
def setup_logger(name : str , log_file : str , level : int = logging.DEBUG) :
    """To setup as many loggers as you want"""

    handler = logging.FileHandler(log_file)
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)

    return logger
# endregion


# region for log folder creation
Path(logs_path).mkdir(parents = True, exist_ok = True)
# endregion


from wplay.utils import browser_config
import time
import os

async def get_all_media():
    cur_path = os.getcwd() + '/wplay/media_images'
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    total_contacts = int(input("Please provide total whatsapp contacts: "))
    loop = round(total_contacts/7)
    images_list =  []

    await page.waitForSelector('#pane-side > div:nth-child(1) > div > div > div:nth-child(1) > div > div > div > div > img')

    for c in range(loop):
        for i in range(1, 18):
            selector = f"#pane-side > div:nth-child(1) > div > div > div:nth-child({i}) > div > div > div > div > img"
            try:
                await page.waitForSelector(selector, timeout=2000)
                image_url = await page.evaluate(f'document.querySelector("{selector}").getAttribute("src")')
                print(f"{c}:{i}-{image_url}")
                if image_url not in images_list:
                    images_list.append(image_url)
            except:
                print("No profile image found")
        await page.evaluate("document.querySelector('#pane-side').scrollBy(0, 500)")
  
    for count in range(len(images_list)):
        try:
            viewSource = await page.goto(images_list[count])
            f = open(os.path.join(cur_path, f'{count}.jpg'), 'wb')
            f.write(await viewSource.buffer())
            f.close()
        except:
            print("Error saving image")
    
    print("Saved all the images to media_folder.")
    time.sleep(5)



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="wplay",
    version="8.0.0",
    install_requires=["python-telegram-bot >= 11.1.0",
                      "datetime >= 4.3",
                      "playsound >= 1.2.2",
                      "argparse >= 1.4.0",
                      "beautifulsoup4 >= 4.8.1",
                      "pyppeteer >= 0.0.25",
                      "whaaaaat>=0.5.2",
                      "prompt_toolkit==1.0.14",
                      "pyfiglet>=0.8.post1",
                      "requests>=2.22.0",
                      "psutil>=5.7.0",
                      "flake8>=3.7.9",
                      "transitions>=0.7.2"
                      ],
    packages=find_packages(),
    description="command line software to play with your WhatsApp",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Rohit Potter, Alexandre Calil",
    author_email="rohitpotter12@gmail.com, alexandrecalilmf@gmail.com",
    license="MIT",
    python_requires=">=3.6",
    url="https://github.com/rpotter12/whatsapp-play/",
    download_url="https://pypi.org/project/wplay/",
    keywords=[
        "whatsapp",
        "whatsapp-cli",
        "whatsapp-chat",
        "message-blast",
        "message-timer",
        "tracker",
        "online tracking",
        "save-chat"
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["wplay = wplay.__main__:main"]},
)



from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from datetime import datetime
import time
import sys

async def schedule_message(target):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        await target_search.search_and_select_target(page, target)
    else:
        await target_select.manual_select_target(page)
    time_ = input("Enter the schedule time in HH:MM:SS format-> ")
    hour, minute, second = time_.split(':')
    current_time = datetime.now()
    delta_hour : int = int(hour) - current_time.hour
    delta_min : int = int(minute) - current_time.minute
    delta_second : int = int(second) - current_time.second
    total_seconds : int = delta_hour*3600 + delta_min*60 + delta_second
    if total_seconds < 0:
        print("Current time is ahead of the scheduled time")
        sys.exit()
    message : list[str] = io.ask_user_for_message_breakline_mode()
    print("Your message is scheduled at : ", time_)
    time.sleep(total_seconds)
    await io.send_message(page, message)



from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import io
from newsapi.newsapi_client import NewsApiClient
import time
import os
from dotenv import load_dotenv

load_dotenv()

newsapi = NewsApiClient(api_key=os.getenv("NEWS_API_KEY"))

async def get_news(target):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        try:
            await target_search.search_and_select_target(page, target)
        except Exception as e:
            print(e)
            await target_search.search_and_select_target_without_new_chat_button(page, target)
    else:
        await target_select.manual_select_target(page)
    
    country = input("Enter your country code (ex: us or in): ")
    while True:
        try:
            news, source = fetch_news(country)
            news_ = f"*{news}* \n Full News :  {source}"
        except Exception as e:
            print("Unable to get the news", e)
        await io.send_message(page, news_)
        time.sleep(900) # Sends news in every 15 min


def fetch_news(country_code):
    headlines = newsapi.get_top_headlines(country=country_code, language='en')
    url = headlines['articles'][0]['url']
    title = headlines['articles'][0]['title']
    return title, url



import time
from pathlib import Path
from datetime import datetime
from playsound import playsound
from wplay.utils import browser_config
from wplay.utils import target_search
from wplay.utils import target_select
from wplay.utils import target_data
from wplay.utils.helpers import data_folder_path
from wplay.utils import Logger
from wplay.utils.helpers import logs_path


#region LOGGER create
logger = Logger.setup_logger('logs',logs_path/'logs.log')
#endregion


async def tracker(target):
    page, _ = await browser_config.configure_browser_and_load_whatsapp()
    if target is not None:
        try:
            target_name = await target_search.search_and_select_target(page, target, hide_groups = True)
        except Exception as e:
            print(e)
            await page.reload()
            target_name = await target_search.search_and_select_target_without_new_chat_button(page,target,hide_groups=True)
    else:
        target_name = await target_select.manual_select_target(page, hide_groups = True)
    Path(data_folder_path / 'tracking_data').mkdir(parents = True, exist_ok = True)
    status_file : str = open(data_folder_path / 'tracking_data' / f'status_{target_name}.txt', 'w').close()
    status_file : str = open(data_folder_path / 'tracking_data' / f'status_{target_name}.txt', 'a')

    is_sound_enabled : bool = True
    last_status : str = 'offline'
    try:
        print(f'Tracking: {target_name}')
        logger.info("Tracking target")
        status_file.write(f'Tracking: {target_name}\n')
        while True:
            status : str = await target_data.get_last_seen_from_focused_target(page)
            if status == 'online':
                is_online : bool = True
            else:
                is_online : bool = False
                status : str = 'offline'
            if last_status != is_online:
                if is_online:
                    try:
                        if is_sound_enabled:
                            playsound('plucky.wav')
                    except:
                        print("Error: Couldn't play the sound.")
                        is_sound_enabled : bool = False
                print(
                    f'{datetime.now().strftime("%d/%m/%Y, %H:%M:%S")}' + f' - Status: {status}'
                )
                status_file.write(
                    f'{datetime.now().strftime("%d/%m/%Y, %H:%M:%S")}' + f' - Status: {status}\n')
            last_status : str = is_online
            time.sleep(0.5)
    except KeyboardInterrupt:
        logger.error("User Pressed Ctrl+C")
    finally:
        status_file.close()
        print(f'\nStatus file saved in: {str(data_folder_path/"tracking_data"/"status_")}{target_name}.txt')
